@@ -5,6 +5,34 @@ import re
 from functools import lru_cache
 from typing import Iterable, List, NamedTuple, Optional, Tuple, Union
 from ..models import Url
+import asyncio
+
+
+
+
+
+
+def _pack(mode: int, flag: bool, number: int) -> bytes:
+    if not (1 <= mode <= 4): raise ValueError("mode должен быть 1..4")
+    if number >= (1 << 61): raise ValueError("number должен быть < 2^61")
+    value = ((mode - 1) & 0b11) << 62
+    value |= (1 << 61) if flag else 0
+    value |= number & ((1 << 61) - 1)
+    return value.to_bytes(8, "big")
+
+def _unpack(data: bytes):
+    if len(data) < 8: raise Exception('len < 8')
+    value = int.from_bytes(data[:8], "big")
+    mode = ((value >> 62) & 0b11) + 1
+    flag = bool((value >> 61) & 1)
+    number = value & ((1 << 61) - 1)
+    return mode, flag, number
+
+
+
+
+
+
 
 
 
@@ -13,10 +41,11 @@ class GNRequest:
         self,
         method: str,
         url: Url,
-        payload: Optional[dict] = None, # msqpack object
+        payload: Optional[dict] = None, # msgpack object
         cookies: Optional[dict] = None, # передаются один раз. сохраняются на сервере в сессии,
         gn_protocol: Optional[str] = None,
-        route: Optional[str] = None
+        route: Optional[str] = None,
+        stream: bool = False
     ):
         self._method = method
         self._url = url
@@ -24,44 +53,214 @@ class GNRequest:
         self._cookies = cookies
         self._gn_protocol = gn_protocol
         self._route = route
+        self._stream = stream
 
         self._url.method = method
 
+        self.user = self.__user(self)
 
-    def serialize(self) -> bytes:
-        """Сериализует объект GNRequest в байтовый формат."""
-        if self._gn_protocol is None:
-            self.setGNProtocol()
+    class __user:
+        def __init__(self, request: GNRequest) -> None:
+            self.__request = request
+            self._data = {}
         
-        if self._route is None:
-            self.setRoute()
+        @property
+        def gwisid(self) -> int:
+            """
+            # ID объекта
 
-        return msgpack.dumps({
-            "method": self._method,
-            "url": str(self._url),
-            "payload": self._payload,
-            "cookies": self._cookies,
-            "gn": {
-                'protocol': self._gn_protocol,
-                'route': self._route
+            Возвращает уникальный идентификатор объекта в системе GW
+
+            Этот идентификатор используется для управления объектами в системе.
+
+            Может использоваться для идентификации пользователя.
+            
+            :return: int
+            """
+            return self._data.get("gwisid", 0)
+        
+        @property
+        def sessionId(self) -> int:
+            """
+            # ID сессии
+
+            Возвращает уникальный идентификатор сессии пользователя в сети GN.
+            
+            Этот идентификатор используется для отслеживания состояния сессии пользователя в системе.
+
+            Может использоваться для идентификации пользователя.
+            
+            :return: int
+            """
+            return self._data.get("session_id", 0)
+        
+        @property
+        def nickname(self) -> str:
+            """
+            # Никнейм объекта
+
+            Возвращает никнейм объекта в системе GW.
+
+            Никнейм используется для идентификации объекта в системе пользователями.
+
+            Может использоваться для идентификации пользователя.
+
+            :return: str
+            """
+            return self._data.get("nickname", "")
+
+        @property
+        def objectType(self) -> int:
+            """
+            # Тип объекта
+
+            Возвращает тип объекта в системе GW.
+            
+            Тип объекта используется для определения роли и функциональности объекта в системе.
+
+            Может использоваться для идентификации пользователя.
+
+            Возможные значения:
+            - 2: Пользователь
+            - 3: Компания
+            - 4: Проект
+            - 5: Продукт
+
+            :return: int
+            """
+            return self._data.get("object_type", 0)
+        
+        @property
+        def viewingType(self) -> int:
+            """
+            # Тип просмотра
+
+            Возвращает тип просмотра объекта в системе GW.
+            Тип просмотра может быть установлен объекту для определения уровня доступа к объекту.
+
+            Возможные значения:
+            - 0: Просмотр доступен только владельцу объекта
+            - 1: Просмотр не ограничен
+            - 2: Просмотр только авторизованным пользователям
+            - 3: Просмотр только для официально подтвержденных пользователей 
+
+            :return: int
+            """
+            return self._data.get("viewing_type", 0)
+
+        @property
+        def description(self) -> str:
+            """
+            # Описание объекта
+
+            Возвращает описание объекта в системе GW.
+            Описание может содержать дополнительную информацию о объекте.
+
+            :return: str
+            """
+            return self._data.get("description", "")
+
+        @property
+        def name(self) -> str:
+            """
+            # Имя объекта
+
+            Возвращает имя объекта в системе GW.
+
+            Имя НЕ может быть использовано для идентификации объекта в системе пользователями.
+
+            Может использоваться для определения объекта только пользователями.
+
+            :return: str
+            """
+            return self._data.get("name", "")
+        
+        @property
+        def owner(self) -> Optional[int]:
+            """
+            # gwisid владельца объекта
+
+            Возвращает уникальный идентификатор владельца объекта в системе GW.
+            Этот идентификатор используется для определения владельца объекта.
+
+            :return: Optional[int]
+            Если владелец не установлен, возвращает None.
+            """
+            return self._data.get("owner", None)
+        
+        @property
+        def officiallyConfirmed(self) -> bool:
+            """
+            # Официально подтвержденный объект
+
+            Возвращает True, если объект официально подтвержден в системе GW.
+            Официально подтвержденные объекты могут иметь дополнительные права и возможности.
+
+            :return: bool
+            """
+            return self._data.get("of_conf", False)
+
+
+    def serialize(self, mode: int = 0) -> bytes:
+        if self._gn_protocol is None: self.setGNProtocol()
+        if self._route is None: self.setRoute()
+        d = {
+            1: self._method,
+            2: str(self._url),
+            3: {
+                1: self._gn_protocol,
+                2: self._route
             }
-        }, use_bin_type=True)
+        }
+        if d[3][2] == 'gn:proxy:request-to-real-server':
+            d[3][2] = True
+        if self._cookies is not None:
+            d[4] = self._cookies
+        if self._payload is not None:
+            d[5] = self._payload
+        if not mode:
+            d[6] = self.stream
+        if self.user._data != {}:
+            d[3][3] = self.user._data
+        blob: bytes = msgpack.dumps(d, use_bin_type=True)
+        return _pack(mode, self.stream, len(blob) + 8) + blob if mode else blob
 
     @staticmethod
-    def deserialize(data: bytes) -> 'GNRequest':
-        """Десериализует байтовый формат в объект GNRequest."""
-        unpacked = msgpack.loads(data, raw=False)
-        _url = Url(unpacked["url"])
+    def deserialize(data: bytes, mode: int = 0) -> 'GNRequest':
+        if mode:
+            if len(data) < 8:
+                raise Exception('len')
+            _mode, stream, length = _unpack(data[:8])
+            if _mode != mode:
+                raise Exception('decrypt error')
+            data = data[8:length]
+        else:
+            stream = None
+        unpacked: dict = msgpack.loads(data, raw=False, strict_map_key=False)
+        _url = Url(unpacked[2])
         if not _url.method:
-            _url.method = unpacked["method"]
-        return GNRequest(
-            method=unpacked["method"],
+            _url.method = unpacked[1]
+        route_ = unpacked.get(3, {}).get(2)
+        if mode:
+            if route_ is True:
+                route_ = 'gn:proxy:request-to-real-server'
+        
+
+        r = GNRequest(
+            method=unpacked[1],
             url=_url,
-            payload=unpacked.get("payload"),
-            cookies=unpacked.get("cookies"),
-            gn_protocol=unpacked.get("gn", {}).get("protocol"),
-            route=unpacked.get("gn", {}).get("route")
+            payload=unpacked.get(5),
+            cookies=unpacked.get(4),
+            stream=stream if stream is not None else unpacked.get(6),
+            gn_protocol=unpacked.get(3, {}).get(1),
+            route=route_
         )
+        r.user._data = unpacked[3].get(3, {})
+        return r
+    @staticmethod
+    def type(data: bytes) -> Tuple[int, bool, int]:
+        return _unpack(data)
+
     @property
     def method(self) -> str:
         """
@@ -114,27 +313,34 @@ class GNRequest:
 
     def setCookies(self, cookies: dict):
         self._cookies = cookies
+        
 
     @property
-    def gn_protocol(self, rStr: bool = False) -> Optional[Union[str, 'GNProtocol']]:
+    def gn_protocol(self) -> Optional['GNProtocol']:
         """
         Возвращает GN протокол
 
         GN протокол используется для подключения к сети GN.
         Если протокол не установлен, возвращает None.
         """
-        if rStr:
-            return self._gn_protocol
         return GNProtocol(self._gn_protocol) if self._gn_protocol else None
+
+    @property
+    def gn_protocol_str(self) -> Optional[str]:
+        """
+        Возвращает GN протокол в виде строки.
+        Если GN протокол не установлен, возвращает None.
+        """
+        return self._gn_protocol
     
     def setGNProtocol(self, gn_protocol: Optional[str] = None):
         """
         Устанавливает GN протокол.
         :param gn_protocol: GN протокол (например, 'gn:tcp:0.1', 'gn:quic',..).
-        Если не указан, используется 'gn:quic'.
+        Если не указан, используется 'gn:quic:real'.
         """
         if gn_protocol is None:
-            gn_protocol = 'gn:quic'
+            gn_protocol = 'gn:quic:real'
         self._gn_protocol = gn_protocol
 
     @property
@@ -156,36 +362,83 @@ class GNRequest:
             route = 'gn:proxy:request-to-real-server'
         self._route = route
 
+    @property
+    def stream(self) -> bool:
+        return self._stream
 
     def __repr__(self):
         return f"<GNRequest [{self._method} {self._url}]>"
     
 class GNResponse:
-    def __init__(self, command: str, payload: Optional[bytes]):
+    def __init__(self, command: str, payload: Optional[dict] = None, stream: bool = False):
         self._command = command
         self._payload = payload
-    def serialize(self) -> bytes:
-        return msgpack.dumps({
-            "command": self._command,
-            "payload": self._payload
-        }, use_bin_type=True)
+        self._stream = stream
+
+    def serialize(self, mode: int = 0) -> bytes:
+        d = {
+            1: self._command
+        }
+
+
+        if d[1] == 'ok':
+            d[1] = True
+
+        if self._payload is not None:
+            d[2] = self._payload
+
+        if not mode:
+            d[3] = self.stream
+
+
+        blob: bytes = msgpack.dumps(d, use_bin_type=True)
+        return _pack(mode, self.stream, len(blob) + 8) + blob if mode else blob
     
     @staticmethod
-    def deserialize(payload: bytes) -> 'GNResponse':
-        unpacked = msgpack.loads(payload, raw=False)
+    def deserialize(data: bytes, mode: int = 0) -> 'GNResponse':
+        if mode:
+            if len(data) < 8:
+                raise Exception('len')
+            
+            _mode, stream, length = _unpack(data[:8])
+
+            if _mode != mode:
+                raise Exception('decrypt error')
+
+            data = data[8:length]
+        else:
+            stream = None
+
+                
+
+        unpacked = msgpack.loads(data, raw=False, strict_map_key=False)
+
+        if mode:
+            cm = unpacked.get(1)
+            if cm is True:
+                cm = 'ok'
+
         return GNResponse(
-            command=unpacked.get("command", 'gn/not_command'),
-            payload=unpacked.get("payload")
+            command=unpacked.get(1, 'gn:no-command'),
+            payload=unpacked.get(2),
+            stream=stream if stream is not None else unpacked.get(3)
         )
+    
+    @staticmethod
+    def type(data: bytes) -> Tuple[int, bool, int]:
+        return _unpack(data)
 
     def command(self) -> str:
         return self._command
 
+    @property
     def payload(self) -> Optional[dict]:
         return self._payload
     
-
-
+    @property
+    def stream(self) -> bool:
+        return self._stream
+    
 
 
 
@@ -436,36 +689,54 @@ class GNProtocol:
 
         def __init__(self, name: str, ver_raw: Optional[str]):
             self._name = name
-            self._ver_raw = ver_raw  # None == last
+            self._ver_raw = ver_raw  # None == 'last'
 
-        # ---------- trivial getters ----------
-        def protocol(self) -> str:
+        # ───────── getters ──────────────────────────────────────────────
+        def protocol(self) -> str:                    # 'proton'
             return self._name
 
-        def version(self) -> str:
+        def version(self) -> str:                     # 'last' / '1.2'
             return self._ver_raw or "last"
 
-        # ---------- hottest path: pattern matching ----------
-        def matches_any(self, patterns: Iterable[str]) -> bool:  # noqa: C901
+        # ───────── hot matcher ──────────────────────────────────────────
+        def matches_any(self, *patterns) -> bool:
+            """
+            Ultra‑fast одноуровневое сравнение.
+
+            • `matches_any("proton")`
+            • `matches_any("proton", "foo:1-2")`
+            • `matches_any(("proton", "foo:1-2"))`
+            """
+            # ---------- нормализация аргументов ----------
+            if len(patterns) == 1 and not isinstance(patterns[0], str):
+                patterns_iter = patterns[0]           # iterable
+            else:
+                patterns_iter = patterns              # *args
+
             nm = self._name
             vr = self._ver_raw
 
-            for p in patterns:
+            # ---------- основной цикл ----------
+            for p in patterns_iter:
                 pat = _compile_leaf_pattern(p)
+
                 # имя
                 if pat.name is not None and pat.name != nm:
                     continue
-                # требуем, чтобы версия была 'last'
+
+                # требуем last
                 if pat.need_last:
                     if vr is not None:
                         continue
                     return True
-                # диапазон/одиночная версия
+
+                # диапазон / одиночная версия / wildcard
                 if pat.ver.contains(vr):
                     return True
+
             return False
 
-        # -------- convenience dunder --------
+        # --------------------------------------------------------------
         def __repr__(self) -> str:
             return f"<Proto {self._name}:{self.version()}>"
 
